//
//  CFRMandelbrotRender.m
//  Mandelbrot
//
//  Created by Renaud Pradenc on 03/02/09.
//  Copyright 2009 Céroce. All rights reserved.
//

#import "CFRMandelbrotRender.h"

#define MAX_ITERATIONS	18

@implementation CFRMandelbrotRender

#pragma mark Gestion de la classe

- (id) init
{
	if(self = [super init])
	{
		largeurBitmap = 400;
		hauteurBitmap = 300;

		largeur = 4.0;
		centre.reel = -0.7;
		centre.imag = 0.0;
	}
	
	return self;
}



#pragma mark Accesseurs
- (void) setLargeurBitmap:(NSUInteger)largeurPixels
{
	largeurBitmap = largeurPixels;	
}

- (void) setHauteurBitmap:(NSUInteger)hauteurPixels
{
	hauteurBitmap = hauteurPixels;	
}


#pragma mark Rendu
/* Renvoie une bitmap de 400x300 pixels, où est représenté l'ensemble de
   Mandelbrot. */
- (NSBitmapImageRep*) bitmapImageRep
{
	// Créer la bitmap
	NSBitmapImageRep* bitmapRep = [[NSBitmapImageRep alloc] initWithBitmapDataPlanes:NULL
																		  pixelsWide:largeurBitmap
																		  pixelsHigh:hauteurBitmap
																	   bitsPerSample:8
																	 samplesPerPixel:1
																			hasAlpha:NO
																			isPlanar:NO
																	  colorSpaceName:NSDeviceWhiteColorSpace
																		 bytesPerRow:0
																		bitsPerPixel:8];
	
	// Calculer l'ensemble de Mandelbrot:
	// Parcourir tous les points de la bitmap
	double x, y;
	for(x = 0; x < largeurBitmap; x++)
	{
		for(y = 0; y < hauteurBitmap; y++)
		{
			// Convertir les coordonnées
			Complexe_t c;
			c = [self complexeAvecCoordBitmapX:x y:y];
			
			// Initialiser z[0]
			Complexe_t z = {0.0, 0.0};
			
			NSUInteger n;
			for(n=0; n < MAX_ITERATIONS; n++)
			{
				// z[n+1] = z[n+1]^2 + c
				z = Additionner(Carre(z), c);
				
				// La suite diverge si |z| > 2
				if(ModuleAuCarre(z) > 4.0)
					break;
			}
			
			// Donner le niveau de gris au pixel
			NSUInteger nuance = n * 255 / MAX_ITERATIONS;
			[bitmapRep setPixel:&nuance atX:x y:y];
		}
	}
	
	[bitmapRep autorelease];
	return bitmapRep;
}

/* Convertit les coordonnées d'un point sur la bitmap en un nombre complexe (coordonnées
   sur le repère mathématique).
*/
- (Complexe_t) complexeAvecCoordBitmapX:(NSUInteger)bitmapX y:(NSUInteger)bitmapY
{
	Complexe_t point;
	
	double hauteur = (largeur/(double)largeurBitmap) * (double)hauteurBitmap;
	
	point.reel = largeur * ((double)bitmapX/(double)largeurBitmap - 0.5) + centre.reel;
	point.imag = hauteur * (0.5 - (double)bitmapY/(double)hauteurBitmap) + centre.imag; 
	
	return point;
}

@end
